'Filename   ATU_RemoteUnit_210.bas
'Version    2.10 for use with PCB V2.00 (11/2012)
'Date       2016-09-13
'Purpose    ATU remote unit, controlled via RS485
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   ATmega 16, MAX485, ULN2803, LCD 16x2 for debugging
'Compiler   BASCOM-AVR 2.0.7.6
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'           Flash used: 2.632 bytes (16%)
'Edit history
'V0.02      2009-07-10
'           first release
'V2.00      2013-01-01
'           LCD disable via jumper @PD.3
'           Fix in Sub SendResponse
'V2.10      2016-09-13
'           RS485 protocoll changed, compatible to ATU_Controller_210.bas

'==============================================================================
'PortA   C-Bank
'        C0             PA0
'        C1             PA1
'        C2             PA2
'        C3             PA3
'        C4             PA4
'        C5             PA5
'        C6             PA6
'        C7             PA7
'PortB   ISP + LCD for Tests
'LCD     RS             PB0
'        E              PB1
'        D5             PB2
'        D7             PB3
'        D6             PB4
'        D4             PB5 with Jumper J1
'ISP     MOSI           PB5
'        MISO           PB6
'        SCK            PB7
'        Reset
'PortC   L-Bank
'        L0             PC0
'        L1             PC1
'        L2             PC2
'        L3             PC3
'        L4             PC4
'        L5             PC5
'        L6             PC6
'        L7             PC7
'PortD   RS485 Communication, High/Low pass, N.C.
'        RXD            PD0
'        TXD            PD1
'        Send/Receive   PD2 (1=Send, 0=Receive)
'        LCD enabled    PD3 Jumper to GND out
'        N.C.           PD4 ... PD6
'        High/Low pass  PD7

'Initial setting of Remote Unit (Relais not set):
'- Low pass
'- All L shortened (Encoder value 000)
'- All C open (Encoder value 000)
'==============================================================================
'FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'In ATMEGA-Data sheets: Bit = 1 -> not set
'                       Bit = 0 -> set
'OCDEN      [ ] (disabled)
'JTAGEN     [ ] (disabled)
'SPIEN      [x] Do not change!
'EESAVE     [ ]
'BOOTSZ     Boot Flash size=128 words (minimal boot size)
'BOOTRST    [ ]
'CKOPT      [ ]
'BODLEVEL   Brown-out detection at VCC=2.7 V
'BODEN      [ ]
'SUT_CKSEL  Ext. Crystal Osc. 8.0- MHz, Start-up time 16k + 64ms

$regfile = "m16def.dat"                           'ATMega16
$crystal = 16000000                               'Crystal 16,0 MHz
$Baud = 9600                                      'Baud rate
$hwstack = 40
$swstack = 40
$framesize = 40
Const txtVers = "V2.10"                           'Version number

'Common Variables, multiply used ----------------------------------------------
Dim bytTmp0 As Byte
Dim bytTmp1 As Byte
Dim wrdTmp0 As Word
Dim lngTmp0 As Long
Dim strTmp6 As String * 6

'Variables Display (for test) -------------------------------------------------
Dim bytDispLC(2) As Byte

'Variables L-/C-Bank ----------------------------------------------------------
Const bytDelta_C = 313                            'Lowest Change of C in 1/100 pF (3.13 pF)
'                                                  'Max. value: 255*313=79,815 (798.15 pF)
Const bytDelta_L = 13                             'Lowest Change of L in 1/100 uH (0.13 uH)
'                                                  'Max. value: 255*13=3,315 (33.15 uH)

'Variables RS485 Communication ------------------------------------------------
Const bytConAdr = &H00                            'ATU Controller  COM address
Const bytATUAdr = &H01                            'ATU remote unit COM address
Const bytATUCall = &HAA                           'Hex code for ATU relais switching
Const bytDummy = &HFF                             'Dummy byte
Const bytFrameLen = 7                             'Length of protocol frame (Bytes)
Const bytFrameLenminus1 = bytFrameLen - 1
Dim bytTRXSend As Byte                            '=1: Send response
Dim bytComErr As Byte                             'communication error
Dim bytBuffCount As Byte                          'Counter for byte buffer
Dim bytCRC8 As Byte                               'CRC8-Checksum
Dim bytBuffInp(bytFrameLen) As Byte               'Input buffer
Dim bytBuffOut(bytFrameLen) As Byte               'Output buffer
Const ON = 1
Const OFF = 0

'Configure Port A (C-Bank) ----------------------------------------------------
DdrA = &B11111111                                 'PortA = Output (C-Bank)
PortA = &B00000000                                'Output low
Port_CBank Alias Porta

'Configure Port B, LCD for test only ------------------------------------------
'Config Lcd = 16 * 2
Config Lcd = 20 * 2
Config Lcdpin = Pin , Db7 = PortB.3 , Db6 = PortB.4 , Db5 = PortB.2
Config Lcdpin = Pin , Db4 = PortB.5 , Rs = PortB.0 , E = PortB.1
Config Lcdbus = 4

'Configure Port C (L-Bank) ----------------------------------------------------
DDRC = &B11111111                                 'PortC = Output (L-Bank)
PORTC = &B00000000                                'Output low
Port_LBank Alias PortC

'Configure Port D (RS485, enable LCD, Hi/Lo pass relais) ----------------------
DdrD = &B10000110                                 'PortD.1, D.2, D.7=output,
'                                                  D.0=RXD, D.1=TXD
'                                                  D.3 jumper in: disable LCD
'                                                  D.4-6 not used (input)
PortD = &B01111000                                'Output low, Pullup D.3-6
bitTransmit Alias PortD.2                         '=1: Transmit, =0: Receive
bitEnableLCD Alias PIND.3                         '=1: enable LCD (jumper out)
bitHT_Pass Alias PortD.7                          '=1: High pass, =0: Low pass

'Subroutines & Functions ------------------------------------------------------
Declare Sub DisplayLC(byVal bytRow As Byte)       'Display L/C & High-/Low pass
Declare Sub ShowRec(byVal bytRow As Byte)         'LCD Test: received message
Declare Sub ShowSnd(byVal bytRow As Byte)         'LCD Test: sent message
Declare Sub ClearLCDLine(byVal bytRow As Byte , byVal bytChars As Byte)

'Enable UART receive interrupt service routine --------------------------------
On URXC RXD_isr                                   'Go to RXD_isr if character is received
Enable URXC

'Initialize data --------------------------------------------------------------
bytBuffCount = 0                                  'Input buffer counter
bitTransmit = OFF                                 'RS485 receive
SREG.7 = 1                                        'Enable all interrupts

Cls
Cursor off
Upperline
Lcd "RemoteUnit " ; txtVers                       'For test if LCD is connected
Lowerline
If bitEnableLCD = 1 Then
   LCD "LCD is enabled"
Else
   LCD "LCD is disabled"
End If
Wait 1
Cls

Do                                                'Main program ***************
   'Evaluate Input buffer
   'RS485 protocoll from ATU controller:
   '1     Address of message destination (bytATUAdr)
   '2     Address of message source  (bytConAdr)
   '3     Command (bytATUCall)
   '4     Value L (0 ... 255)
   '5     Value C (0 ... 255)
   '6     Value H/L (1 = High pass, 0 = Low pass)
   '7     Checksum of bytes 1 to 6 (CRC8, 1 byte)

   If bytBuffCount = bytFrameLen Then             'input buffer full
      bytComErr = 21                              'Msg not acknoledged (NAK)
      bytTRXSend = 0                              'Do not send response
      IF bitEnableLCD = 1 Then
         Call ShowRec(1)
      End If
      If bytBuffInp(1) = bytATUAdr Then           'It's for me, the remote unit
         bytTRXSend = 1                           'Send response
         bytCRC8 = CRC8(bytBuffInp(1) , bytFrameLenMinus1)
         If bytCRC8 = bytBuffInp(7) Then          'Checksum is OK
            If bytBuffInp(2) = bytConAdr Then     'It's from ATU controller
               If bytBuffInp(3) = bytATUCall Then 'It's set relais command
                  'All OK, set Relais
                  bytDispLC(1) = bytBuffInp(4)    'Value L
                  bytDispLC(2) = bytBuffInp(5)    'Value C
                  bitHT_Pass = bytBuffInp(6)      'High-/low pass
                  'IF bitEnableLCD = 1 Then
                     'Call DisplayLC(1)
                  'End If
                  Port_LBank = bytDispLC(1)       'Set Relais L
                  Port_CBank = bytDispLC(2)       'Set Relais C
                  bytComErr = 6                   'Msg acknoledged (ACK)
               End If
            End If
         End If
      Else                                        'It's not for me, the remote unit

      End If

      If bytTRXSend = 1 Then
         'Send response to ATU controller
         'RS485 protocoll to ATU remote unit:
         '1     Address of message destination (bytConAdr)
         '2     Address of message source  (bytATUAdr)
         '3     Command (bytATUCall)
         '4     ACK  (6), Acknoledge , CRC OK, message confirmed or
         '      NAK (21), Non-ACK,     CRC not OK, message not confirmed
         '5     --- (bytDummy)
         '6     --- (bytDummy)
         '7     Checksum of bytes 1 to 6 (CRC8, 1 byte)
         UCSRA.6 = 1                              'Clear UART Transmit Complete bit
         bitTransmit = ON
         Waitms 2                                 'Wait for MAX485 to switch
         bytBuffOut(1) = bytConAdr
         bytBuffOut(2) = bytATUAdr
         bytBuffOut(3) = bytATUCall
         bytBuffOut(4) = bytComErr
         bytBuffOut(5) = bytDummy
         bytBuffOut(6) = bytDummy
         bytCRC8 = CRC8(bytBuffOut(1) , bytFrameLenMinus1)
         bytBuffOut(7) = bytCRC8
         PrintBin bytBuffOut(1) ; bytFrameLen     'Send array bytBuffOut
         Do
         Loop Until UCSRA.6 = 1                   'Wait until all bytes are sent
         bitTransmit = OFF                        'ready for receive
         Waitms 2                                 'Wait for MAX485 to switch
         IF bitEnableLCD = 1 Then
            Call ShowSnd(2)
         End If
      End If
      bytBuffCount = 0                            'input buffer empty
   End If
   Idle                                           'Put the processor into the idle mode
                                                   'saves ~10 mA current
Loop                                              'End of main program ****************


'==============================================================================
'URXC receive interrupt service routine
'Output:  bytBuffInp(1...byt_Count), Input buffer
'         byt_Count, current length of input buffer

RXD_isr:
   Incr bytBuffCount
   bytBuffInp(bytBuffCount) = UDR                 'Put next byte into buffer
Return

'==============================================================================
Sub ShowRec(byVal bytRow As Byte)
   'Display received message for test if LCD is connected
   Locate bytRow , 1
   Lcd SPC(16)
   Locate bytRow , 1
   'STX, Adr and Len
   Lcd bytBuffInp(1) ; " " ; bytBuffInp(2) ; " " ; bytBuffInp(3) ; " "
   'Value L
   strTmp6 = str(bytBuffInp(4))
   strTmp6 = Format(strTmp6 , "000")
   Lcd strTmp6 ; " "
   'Value C
   strTmp6 = str(bytBuffInp(5))
   strTmp6 = Format(strTmp6 , "000")
   Lcd strTmp6 ; " "
   'High/low pass
   Lcd bytBuffInp(6)

End Sub

'==============================================================================
Sub ShowSnd(byVal bytRow As Byte)
   'Display sent message for test if LCD is connected
   Locate bytRow , 1
   Lcd SPC(16)
   Locate bytRow , 1
   'STX, Adr, Len
   Lcd bytBuffOut(1) ; " " ; bytBuffOut(2) ; " " ; bytBuffOut(3) ; " "
   'Acknoledge
   strTmp6 = str(bytBuffOut(4))
   strTmp6 = Format(strTmp6 , "000")
   Lcd strTmp6

End Sub

'==============================================================================
Sub DisplayLC(byVal bytRow As Byte)
   'Display L/C & High-/Low pass in 1. row
   'Input:
   'bytDispLC(1)   Encoder value L
   'bytDispLC(2)   Encoder value C
   'bytDispLC(3)   =1: L active, =2: C active, =3: H/L pass active
   'bitHT_Pass     =1: High pass, =0: Low pass

   Locate bytRow , 1
   Lcd "L"
   Locate bytRow , 9
   Lcd "C"

   wrdTmp0 = bytDispLC(1) * bytDelta_L            'L in 1/100 uH
   strTmp6 = str(wrdTmp0)
   strTmp6 = Format(strTmp6 , " 0.00")            'L in uH
   bytTmp0 = Len(strTmp6)
   bytTmp0 = bytTmp0 - 1
   strTmp6 = Left(strTmp6 , bytTmp0)              'Cut last digt: xx.x uH
   Locate bytRow , 3
   Lcd strTmp6 ; "u "
   lngTmp0 = bytDispLC(2) * bytDelta_C            'C in 1/100 pF
   strTmp6 = str(lngTmp0)
   strTmp6 = Format(strTmp6 , "  0.00")           'L in pF
   bytTmp0 = Len(strTmp6)
   bytTmp0 = bytTmp0 - 3
   strTmp6 = Left(strTmp6 , bytTmp0)              'cut last 2 digits: xxx pF
   Locate bytRow , 11
   Lcd strTmp6 ; "p"
   Locate bytRow , 16
   If bitHT_Pass = 1 Then                         'High pass
      Lcd "H"
   Else                                           'Low pass
      Lcd "L"
   End If
End Sub

'==============================================================================
Sub ClearLCDLine(byVal bytRow As Byte , byVal bytChars As Byte)
   'Clear LCD bytChars characters in line bytRow

   Locate bytRow , 1
   LCD SPC(bytChars)
   Locate bytRow , 1

End Sub

'==============================================================================
End